@@ -50,7 +50,7 @@ def open_uri(uri, mode, transport_params):
 
 
 def open(uri, mode, kerberos=False, user=None, password=None, cert=None,
-         headers=None, timeout=None, buffer_size=DEFAULT_BUFFER_SIZE):
+         headers=None, timeout=None, session=None, buffer_size=DEFAULT_BUFFER_SIZE):
     """Implement streamed reader from a web site.
 
     Supports Kerberos and Basic HTTP authentication.
@@ -73,6 +73,9 @@ def open(uri, mode, kerberos=False, user=None, password=None, cert=None,
         Any headers to send in the request. If ``None``, the default headers are sent:
         ``{'Accept-Encoding': 'identity'}``. To use no headers at all,
         set this variable to an empty dict, ``{}``.
+    session: object, optional
+        The requests Session object to use with http get requests.
+        Can be used for OAuth2 clients.
     buffer_size: int, optional
         The buffer size to use when performing I/O.
 
@@ -86,7 +89,7 @@ def open(uri, mode, kerberos=False, user=None, password=None, cert=None,
         fobj = SeekableBufferedInputBase(
             uri, mode, buffer_size=buffer_size, kerberos=kerberos,
             user=user, password=password, cert=cert,
-            headers=headers, timeout=timeout,
+            headers=headers, session=session, timeout=timeout,
         )
         fobj.name = os.path.basename(urllib.parse.urlparse(uri).path)
         return fobj
@@ -95,16 +98,23 @@ def open(uri, mode, kerberos=False, user=None, password=None, cert=None,
 
 
 class BufferedInputBase(io.BufferedIOBase):
+    response = None  # so `closed` property works in case __init__ fails and __del__ is called
+
     def __init__(self, url, mode='r', buffer_size=DEFAULT_BUFFER_SIZE,
                  kerberos=False, user=None, password=None, cert=None,
-                 headers=None, timeout=None):
+                 headers=None, session=None, timeout=None):
+
+        self.url = url
+        self.cert = cert
+        self.session = session or requests
+
         if kerberos:
             import requests_kerberos
-            auth = requests_kerberos.HTTPKerberosAuth()
+            self.auth = requests_kerberos.HTTPKerberosAuth()
         elif user is not None and password is not None:
-            auth = (user, password)
+            self.auth = (user, password)
         else:
-            auth = None
+            self.auth = None
 
         self.buffer_size = buffer_size
         self.mode = mode
@@ -116,10 +126,10 @@ class BufferedInputBase(io.BufferedIOBase):
 
         self.timeout = timeout
 
-        self.response = requests.get(
-            url,
-            auth=auth,
-            cert=cert,
+        self.response = self.session.get(
+            self.url,
+            auth=self.auth,
+            cert=self.cert,
             stream=True,
             headers=self.headers,
             timeout=self.timeout,
@@ -128,23 +138,24 @@ class BufferedInputBase(io.BufferedIOBase):
         if not self.response.ok:
             self.response.raise_for_status()
 
-        self._read_iter = self.response.iter_content(self.buffer_size)
+        self._read_iter = iter(lambda: self.response.raw.read(self.buffer_size), b"")
         self._read_buffer = bytebuffer.ByteBuffer(buffer_size)
         self._current_pos = 0
 
-        #
-        # This member is part of the io.BufferedIOBase interface.
-        #
-        self.raw = None
-
     #
     # Override some methods from io.IOBase.
     #
     def close(self):
         """Flush and close this stream."""
         logger.debug("close: called")
-        self.response = None
-        self._read_iter = None
+        if not self.closed:
+            self.response = None
+            self._read_iter = None
+            self._read_buffer = None
+
+    @property
+    def closed(self):
+        return self.response is None
 
     def readable(self):
         """Return True if the stream can be read from."""
@@ -217,7 +228,7 @@ class SeekableBufferedInputBase(BufferedInputBase):
 
     def __init__(self, url, mode='r', buffer_size=DEFAULT_BUFFER_SIZE,
                  kerberos=False, user=None, password=None, cert=None,
-                 headers=None, timeout=None):
+                 headers=None, session=None, timeout=None):
         """
         If Kerberos is True, will attempt to use the local Kerberos credentials.
         If cert is set, will try to use a client certificate
@@ -225,33 +236,7 @@ class SeekableBufferedInputBase(BufferedInputBase):
 
         If none of those are set, will connect unauthenticated.
         """
-        self.url = url
-
-        if kerberos:
-            import requests_kerberos
-            self.auth = requests_kerberos.HTTPKerberosAuth()
-        elif user is not None and password is not None:
-            self.auth = (user, password)
-        else:
-            self.auth = None
-
-        if headers is None:
-            self.headers = _HEADERS.copy()
-        else:
-            self.headers = headers
-
-        self.cert = cert
-        self.timeout = timeout
-
-        self.buffer_size = buffer_size
-        self.mode = mode
-        self.response = self._partial_request()
-
-        if not self.response.ok:
-            self.response.raise_for_status()
-
-        logger.debug('self.response: %r, raw: %r', self.response, self.response.raw)
-
+        super().__init__(url, mode, buffer_size, kerberos, user, password, cert, headers, session, timeout)
         self.content_length = int(self.response.headers.get("Content-Length", -1))
         #
         # We assume the HTTP stream is seekable unless the server explicitly
@@ -261,15 +246,6 @@ class SeekableBufferedInputBase(BufferedInputBase):
         #
         self._seekable = self.response.headers.get("Accept-Ranges", "").lower() != "none"
 
-        self._read_iter = self.response.iter_content(self.buffer_size)
-        self._read_buffer = bytebuffer.ByteBuffer(buffer_size)
-        self._current_pos = 0
-
-        #
-        # This member is part of the io.BufferedIOBase interface.
-        #
-        self.raw = None
-
     def seek(self, offset, whence=0):
         """Seek to the specified position.
 
@@ -311,7 +287,7 @@ class SeekableBufferedInputBase(BufferedInputBase):
             response = self._partial_request(new_pos)
             if response.ok:
                 self.response = response
-                self._read_iter = self.response.iter_content(self.buffer_size)
+                self._read_iter = iter(lambda: self.response.raw.read(self.buffer_size), b"")
                 self._read_buffer.empty()
             else:
                 self.response = None
@@ -329,15 +305,16 @@ class SeekableBufferedInputBase(BufferedInputBase):
         raise io.UnsupportedOperation
 
     def _partial_request(self, start_pos=None):
+        headers = self.headers.copy()
         if start_pos is not None:
-            self.headers.update({"range": smart_open.utils.make_range_string(start_pos)})
+            headers["range"] = smart_open.utils.make_range_string(start_pos)
 
-        response = requests.get(
+        response = self.session.get(
             self.url,
             auth=self.auth,
             stream=True,
             cert=self.cert,
-            headers=self.headers,
+            headers=headers,
             timeout=self.timeout,
         )
         return response
