@@ -524,20 +524,22 @@ class _SeekableRawReader(object):
             start = max(0, offset)
         elif whence == constants.WHENCE_CURRENT:
             start = max(0, offset + self._position)
-        else:
+        elif whence == constants.WHENCE_END:
             stop = max(0, -offset)
 
         #
         # If we can figure out that we've read past the EOF, then we can save
         # an extra API call.
         #
-        if start is None and stop == 0 and self._content_length is None:
-            # seek(0, WHENCE_END) seeks straight to EOF: make a minimal request to populate _content_length
-            self._open_body(start=0, stop=0)
-            self.close()
-            reached_eof = True
-        elif self._content_length is None:
-            reached_eof = False
+        if self._content_length is None:  # _open_body has not been called yet
+            if start is None and stop == 0:
+                # seek(0, WHENCE_END) seeks straight to EOF:
+                # make a minimal request to populate _content_length
+                self._open_body(start=0, stop=0)
+                self.close()
+                reached_eof = True
+            else:
+                reached_eof = False
         elif start is not None and start >= self._content_length:
             reached_eof = True
         elif stop == 0:
@@ -785,11 +787,6 @@ class Reader(io.BufferedIOBase):
         self._line_terminator = line_terminator
         self._seek_initialized = False
 
-        #
-        # This member is part of the io.BufferedIOBase interface.
-        #
-        self.raw = None
-
         if not defer_seek:
             self.seek(0)
 
@@ -1019,11 +1016,6 @@ class MultipartWriter(io.BufferedIOBase):
         self._total_parts = 0
         self._parts: list[dict[str, object]] = []
 
-        #
-        # This member is part of the io.BufferedIOBase interface.
-        #
-        self.raw = None  # type: ignore[assignment]
-
     def flush(self):
         pass
 
